@@ -186,19 +186,6 @@ You can pass your own arguments - and overwrite it - with
 
     dbi_params => { PrintError => 1, AutoCommit => 0 }
 
-=item B<use_ping>
-
-By default a short statement is executed to check if the connection
-is alive because ping() doesn't work with all DBD properly.
-
-    select 1
-
-or for oracle
-
-    select 1 from dual
-
-If you want to use the ping() function instead set C<use_ping> to 1.
-
 =item B<debug>
 
 With this option it's possible to enable debugging. The information can be
@@ -229,6 +216,20 @@ Log a message to the database.
         time    => "2008-10-10 10:12:23",
     );
 
+Or you can connect to the database yourself. You should
+notice that if the database connection lost then the
+logger can't re-connect to the database and would return
+an error. You C<dbi_handle> at your own risk.
+
+    my $dbh = DBI->connect(...);
+
+    my $db = Log::Handler::Output::DBI->new(
+        dbi_handle => $dbh,
+        table      => "messages",
+        columns    => [ qw/level ctime message/ ],
+        values     => [ qw/%level %time %message/ ],
+    );
+
 =head2 connect()
 
 Connect to the database.
@@ -272,7 +273,7 @@ Jonny Schulz <jschulz.cpan(at)bloonix.de>.
 
 =head1 COPYRIGHT
 
-Copyright (C) 2007-2010 by Jonny Schulz. All rights reserved.
+Copyright (C) 2007-2009 by Jonny Schulz. All rights reserved.
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
@@ -327,7 +328,7 @@ sub log {
         return $self->_raise_error("DBI execute error: ".DBI->errstr);
     }
 
-    if (!$self->{persistent}) {
+    if (!$self->{persistent} && !$self->{dbi_handle}) {
         $self->disconnect or return undef;
     }
 
@@ -352,8 +353,19 @@ sub connect {
         warn "Connect to the database: $self->{cstr}->[0] ...";
     }
 
-    my $dbh = DBI->connect(@{$self->{cstr}})
-        or return $self->_raise_error("DBI connect error: ".DBI->errstr);
+    my $dbh;
+
+    if ($self->{dbi_handle}) {
+        # If db ping failed and dbi_handle and dbi is set
+        # then it seems that the database is down.
+        if ($self->{dbi}) {
+            return $self->_raise_error("dbi_handle - lost connection");
+        }
+        $dbh = $self->{dbi_handle};
+    } else {
+        $dbh = DBI->connect(@{$self->{cstr}})
+            or return $self->_raise_error("DBI connect error: ".DBI->errstr);
+    }
 
     my $sth = $dbh->prepare($self->{statement})
         or return $self->_raise_error("DBI prepare error: ".$dbh->errstr);
@@ -566,7 +578,7 @@ sub _validate {
         $options{statement} .= ")";
     }
 
-    if ($options{driver} =~ /oracle/i) {
+    if ($options{driver} && $options{driver} =~ /oracle/i) {
         $options{pingstmt} = "select 1 from dual";
     } else {
         $options{pingstmt} = "select 1";
